(defparameter *one-node* nil)
(defun make-print-problem (n colors probnum output-directory which)

    (loop for i from 1 to probnum
          with problem and problem-file
          do   (let* ((node-syms (make-nodes n))
                      (graph (make-graph node-syms colors)))
                 (ecase which
                   (:pddl 
                    (setf problem (make-pddl-problem node-syms colors graph))
                    (setf problem-file (format nil "~a/prob~a.pddl"
                                         output-directory
                                         i))
                    (with-open-file (str problem-file
                                         :direction :output
                                         :if-exists :supersede
                                         :if-does-not-exist :create)
                      (pprint problem str)))
                   
                   (:shop2
                    (setf problem (make-shop2-problem node-syms colors))
                    (setf problem-file (format nil "~a/prob~a.lisp"
                                         output-directory
                                         i))
                    (with-open-file (str problem-file
                                         :direction :output
                                         :if-exists :supersede
                                         :if-does-not-exist :create)
                      (pprint '(in-package :shop2-user) str)
                      (pprint problem str)))))))


(defun make-pddl-problem (node-syms colors graph)
  `(define (problem prob)
       (:domain graph-coloring)
     (:objects ,@node-syms)
     (:init 
      ,@(loop for node in node-syms
              collect `(node ,node))
      ,@graph
      (at ;,(first node-syms)) 
       ,(random-select (subseq node-syms 0 20))) 
      )
     (:goal
      (and (at ,(first (last node-syms))
           )))))

(defun make-shop2-problem (node-syms colors)
  (let ((graph (make-graph node-syms colors)))

    `(defproblem prob graph-coloring
       (
      ,@(loop for node in node-syms
              collect `(node ,node))
        ,@graph
        (at ,(whereis graph))
      )
     (
      (travel ,(whereis graph))))))

#|
(defun make-nodes (n)
  (let (node-set)
    (loop until (= (length node-set) n)
          do (push (intern (gensym (format nil "N~a" (random 100)))) node-set)
             (setf node-set (remove-duplicates node-set)))
    node-set))
|#
(defun make-nodes (n)
  (loop for i from 1 to n
        collect (intern (format nil "N~a" i))))

(defun random-select (list)
  (nth (random (length list)) list))

#| Good, random, by we may not want random...
(defun make-graph (nodes colors)
  (let ((closed (list (random-select nodes))))
    (loop until (equal-lists closed nodes)
          as node = (random-select nodes)
          as parent = (random-select closed)
          as color = (random-select colors)
          collect `(,color ,parent ,node) into edges
          do (push node closed)
          finally (return edges))))
|#

#|
(defun make-graph (nodes colors)
  (let ((closed (list (first nodes)))) ; (random-select nodes))))
    (loop until (equal-lists closed nodes)
          as node = (random-select nodes)
          as parent = (first closed) ; (random-select closed)
          as color = (random-select colors)
          collect `(,color ,parent ,node) into edges
          do (push node closed)
          finally (return edges))))
|#

;;; LOONG plans, different goals for NCD to give diversity values >
;;; 0.8. You need mix them with shorter plans, random closed. 
;;; AD gives 1.0 for everything, shorter plans, same color, no, random
;;; closed,first closed, no gensym, same goals  for it to
;;; give diversity values < 0.33
(defun make-graph (nodes colors)
  (let ((closed (list (first nodes)))) ; (random-select nodes))))
    (loop until (equal-lists closed nodes)
          for i from 1 
          as node = (nth i nodes) 
;         as node = (random-select nodes)
          as parent = (first closed) 
;         as parent = (random-select closed)
          as color = (random-select colors)
;         as color = (first colors)
          collect `(,color ,parent ,node) into edges
          do (push node closed)
          finally (return edges))))

(defun equal-lists (list1 list2)
  (not (or (set-difference list2 list1)
           (set-difference list1 list2))))

(defun whereis (graph)
  (let ((edge (random-select graph)))
    (third edge)))

#|  
(defun whereis (graph
                &key (goal-bias 0.0))
  (let ((edge (random-select graph)))
    (if (= goal-bias 0.0)
      (third edge)
      ;; Otherwise, flip a coin and based on the result either use the
      ;; new node or use the previous one.
      (let ((result (random 1.0)))
        (if (> result goal-bias)
          (third edge)
          *one-node*)))))
|#
